@@ -1,7 +1,9 @@
 from sqlalchemy import Column, Date, DateTime, Float, Integer, String, distinct, func
 from sqlalchemy.ext.declarative import declarative_base
 
-from recipe import Dimension, Metric, Shelf, get_oven
+from recipe import Dimension, Metric, Filter, Shelf, get_oven
+from datetime import date
+from dateutil.relativedelta import relativedelta
 
 oven = get_oven("sqlite://")
 Base = declarative_base(bind=oven.engine)
@@ -20,6 +22,22 @@ oven.engine.execute(
     "insert into foo values ('hi', 'there', 5, '2005-01-01', '2005-01-01 12:15:00'), ('hi', 'fred', 10, '2015-05-15', '2013-10-15 05:20:10')"
 )
 
+
+# Test dynamic date filtering
+TABLEDEF = """
+        CREATE TABLE IF NOT EXISTS datetester
+        (dt date,
+         count integer);
+"""
+
+oven.engine.execute(TABLEDEF)
+start_dt = date(date.today().year, date.today().month, 1)
+# Add dates for the 50 months around the current date
+for offset_month in range(-50, 50):
+    dt = start_dt + relativedelta(months=offset_month)
+    oven.engine.execute("insert into datetester values ('{}', 1)".format(dt))
+
+
 # Create a table for testing summarization
 TABLEDEF = """
         CREATE TABLE IF NOT EXISTS scores
@@ -341,6 +359,13 @@ class Census(Base):
     __table_args__ = {"extend_existing": True}
 
 
+class DateTester(Base):
+    dt = Column("dt", Date(), primary_key=True)
+    count = Column("count", Integer())
+    __tablename__ = "datetester"
+    __table_args__ = {"extend_existing": True}
+
+
 class StateFact(Base):
     id = Column("id", String(), primary_key=True)
     name = Column("name", String())
@@ -415,6 +440,7 @@ census_shelf = Shelf(
         "pop2000": Metric(func.sum(Census.pop2000)),
         "pop2000_sum": Metric(func.sum(Census.pop2000), summary_aggregation=func.sum),
         "pop2008": Metric(func.sum(Census.pop2008)),
+        "filter_all": Filter(1 == 0),
     }
 )
 
