@@ -6,13 +6,15 @@ Test recipes built from yaml files in the ingredients directory.
 import os
 
 from datetime import date
+from dateutil.relativedelta import relativedelta
 import pytest
 from six import text_type
-from tests.test_base import Census, MyTable, oven, ScoresWithNulls
+from tests.test_base import Census, MyTable, oven, ScoresWithNulls, DateTester
 
 from recipe import (
     AutomaticFilters,
     BadIngredient,
+    InvalidIngredient,
     Recipe,
     Shelf,
     BadRecipe,
@@ -274,7 +276,17 @@ FROM census"""
            WHEN (census.state IN ('Vermont',
                                   'New Hampshire')) THEN 'northeast'
            WHEN (census.age < 2) THEN 'babies'
-           WHEN (census.age < 13) THEN 'children'
+           WHEN (census.age IN (2,
+                                3,
+                                4,
+                                5,
+                                6,
+                                7,
+                                8,
+                                9,
+                                10,
+                                11,
+                                12)) THEN 'children'
            WHEN (census.age < 20) THEN 'teens'
            ELSE 'oldsters'
        END AS mixed_buckets,
@@ -282,7 +294,17 @@ FROM census"""
            WHEN (census.state IN ('Vermont',
                                   'New Hampshire')) THEN 0
            WHEN (census.age < 2) THEN 1
-           WHEN (census.age < 13) THEN 2
+           WHEN (census.age IN (2,
+                                3,
+                                4,
+                                5,
+                                6,
+                                7,
+                                8,
+                                9,
+                                10,
+                                11,
+                                12)) THEN 2
            WHEN (census.age < 20) THEN 3
            ELSE 9999
        END AS mixed_buckets_order_by,
@@ -377,6 +399,46 @@ oldsters,9999,4567879,oldsters
 teens,2,614548,teens
 children,1,948240,children
 babies,0,164043,babies
+""",
+        )
+
+    def test_census_buckets_nolabel(self):
+        """If not default label is provided, buckets default to "Not found" """
+        shelf = self.validated_shelf("census.yaml", Census)
+        recipe = (
+            Recipe(shelf=shelf, session=self.session)
+            .dimensions("age_buckets_nolabel")
+            .metrics("pop2000")
+            .order_by("age_buckets_nolabel")
+        )
+        assert (
+            recipe.to_sql()
+            == """SELECT CASE
+           WHEN (census.age < 2) THEN 'babies'
+           WHEN (census.age < 13) THEN 'children'
+           WHEN (census.age < 20) THEN 'teens'
+           ELSE 'Not found'
+       END AS age_buckets_nolabel,
+       CASE
+           WHEN (census.age < 2) THEN 0
+           WHEN (census.age < 13) THEN 1
+           WHEN (census.age < 20) THEN 2
+           ELSE 9999
+       END AS age_buckets_nolabel_order_by,
+       sum(census.pop2000) AS pop2000
+FROM census
+GROUP BY age_buckets_nolabel,
+         age_buckets_nolabel_order_by
+ORDER BY age_buckets_nolabel_order_by,
+         age_buckets_nolabel"""
+        )
+        self.assert_recipe_csv(
+            recipe,
+            """age_buckets_nolabel,age_buckets_nolabel_order_by,pop2000,age_buckets_nolabel_id
+babies,0,164043,babies
+children,1,948240,children
+teens,2,614548,teens
+Not found,9999,4567879,Not found
 """,
         )
 
@@ -1074,6 +1136,19 @@ Vermont,0.4374284968466102,The Green Mountain State,Vermont
 """,
         )
 
+    def test_shelf_with_invalidingredient(self):
+        """Build a recipe using a shelf that uses field references """
+        shelf = self.validated_shelf("census.yaml", Census)
+        assert isinstance(shelf["baddim"], InvalidIngredient)
+        recipe = (
+            Recipe(shelf=shelf, session=self.session)
+            .dimensions("baddim")
+            .metrics("pop2000")
+        )
+        # Trying to run the recipe raises an exception with the bad ingredient details
+        with pytest.raises(BadIngredient):
+            recipe.to_sql()
+
     def test_complex_census_from_validated_yaml_math(self):
         """Build a recipe that uses complex definitions dimensions and
         metrics """
@@ -1146,3 +1221,452 @@ OFFSET 0"""
 11168,Tennessee,77.0,11168
 """,
         )
+
+    def test_is(self):
+        """Test fields using is"""
+        shelf = self.validated_shelf("ingredients1.yaml", MyTable)
+        recipe = (
+            Recipe(shelf=shelf, session=self.session)
+            .metrics("dt_test")
+            .dimensions("first")
+        )
+        assert (
+            recipe.to_sql()
+            == """SELECT foo.first AS first,
+       sum(CASE
+               WHEN (foo.birth_date IS NULL) THEN foo.age
+               ELSE 1
+           END) AS dt_test
+FROM foo
+GROUP BY first"""
+        )
+        self.assert_recipe_csv(
+            recipe,
+            """first,dt_test,first_id
+hi,2,hi
+""",
+        )
+
+    def test_intelligent_date(self):
+        """Test intelligent dates like `date is last year`"""
+        shelf = self.validated_shelf("ingredients1.yaml", MyTable)
+        recipe = (
+            Recipe(shelf=shelf, session=self.session)
+            .metrics("intelligent_date_test")
+            .dimensions("first")
+        )
+        from datetime import date
+        from dateutil.relativedelta import relativedelta
+
+        today = date.today()
+        start_dt = date(today.year - 1, 1, 1)
+        end_dt = start_dt + relativedelta(years=1, days=-1)
+        assert (
+            recipe.to_sql()
+            == """SELECT foo.first AS first,
+       sum(CASE
+               WHEN (foo.birth_date BETWEEN '{}' AND '{}') THEN foo.age
+               ELSE 2
+           END) AS intelligent_date_test
+FROM foo
+GROUP BY first""".format(
+                start_dt, end_dt
+            )
+        )
+        self.assert_recipe_csv(
+            recipe,
+            """first,intelligent_date_test,first_id
+hi,4,hi
+""",
+        )
+
+
+class TestParsedSQLGeneration(object):
+    """More tests of SQL generation on complex parsed expressions """
+
+    def setup(self):
+        self.session = oven.Session()
+
+    def create_shelf(self, config, selectable):
+        return Shelf.from_validated_yaml(config, selectable)
+
+    def assert_recipe_csv(self, recipe, csv_text):
+        assert recipe.dataset.export("csv", lineterminator=text_type("\n")) == csv_text
+
+    def test_complex_field(self):
+        """Test parsed field definitions that use math, field references and more"""
+        shelf = self.create_shelf(
+            """
+_version: 2
+username:
+    kind: Dimension
+    field: username
+count_star:
+    kind: Metric
+    field: "count(*)"
+total_nulls:
+    kind: Metric
+    field: "count_distinct(if(score IS NULL, username))"
+chip_nulls:
+    kind: Metric
+    field: 'sum(if(score IS NULL and username = \"chip\",1,0))'
+user_null_counter:
+    kind: Metric
+    field: 'if(username IS NULL, 1, 0)'
+chip_or_nulls:
+    kind: Metric
+    field: 'sum(if(score IS NULL OR (username = \"chip\"),1,0))'
+simple_math:
+    kind: Metric
+    field: "@count_star +  @total_nulls   + @chip_nulls"
+refs_division:
+    kind: Metric
+    field: "@count_star / 100.0"
+refs_as_denom:
+    kind: Metric
+    field: "12 / @count_star"
+math:
+    kind: Metric
+    field: "(@count_star / @count_star) + (5.0 / 2.0)"
+parentheses:
+    kind: Metric
+    field: "@count_star / (@count_star + (12.0 / 2.0))"
+""",
+            ScoresWithNulls,
+        )
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("count_star")
+        assert (
+            recipe.to_sql()
+            == """SELECT count(*) AS count_star
+FROM scores_with_nulls"""
+        )
+        self.assert_recipe_csv(recipe, "count_star\n6\n")
+
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("total_nulls")
+        assert (
+            recipe.to_sql()
+            == """SELECT count(DISTINCT CASE
+                          WHEN (scores_with_nulls.score IS NULL) THEN scores_with_nulls.username
+                      END) AS total_nulls
+FROM scores_with_nulls"""
+        )
+        self.assert_recipe_csv(recipe, "total_nulls\n3\n")
+
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("chip_nulls")
+        assert (
+            recipe.to_sql()
+            == """SELECT sum(CASE
+               WHEN (scores_with_nulls.score IS NULL
+                     AND scores_with_nulls.username = 'chip') THEN 1
+               ELSE 0
+           END) AS chip_nulls
+FROM scores_with_nulls"""
+        )
+        self.assert_recipe_csv(recipe, "chip_nulls\n1\n")
+
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("chip_or_nulls")
+        assert (
+            recipe.to_sql()
+            == """SELECT sum(CASE
+               WHEN (scores_with_nulls.score IS NULL
+                     OR scores_with_nulls.username = 'chip') THEN 1
+               ELSE 0
+           END) AS chip_or_nulls
+FROM scores_with_nulls"""
+        )
+        self.assert_recipe_csv(recipe, "chip_or_nulls\n5\n")
+
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("user_null_counter")
+        assert (
+            recipe.to_sql()
+            == """SELECT sum(CASE
+               WHEN (scores_with_nulls.username IS NULL) THEN 1
+               ELSE 0
+           END) AS user_null_counter
+FROM scores_with_nulls"""
+        )
+
+        self.assert_recipe_csv(recipe, "user_null_counter\n0\n")
+
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("simple_math")
+        assert (
+            recipe.to_sql()
+            == """SELECT count(*) + count(DISTINCT CASE
+                                     WHEN (scores_with_nulls.score IS NULL) THEN scores_with_nulls.username
+                                 END) + sum(CASE
+                                                WHEN (scores_with_nulls.score IS NULL
+                                                      AND scores_with_nulls.username = 'chip') THEN 1
+                                                ELSE 0
+                                            END) AS simple_math
+FROM scores_with_nulls"""
+        )
+        self.assert_recipe_csv(recipe, "simple_math\n10\n")
+
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("refs_division")
+        assert (
+            recipe.to_sql()
+            == """SELECT CAST(count(*) AS FLOAT) / 100.0 AS refs_division
+FROM scores_with_nulls"""
+        )
+        self.assert_recipe_csv(recipe, "refs_division\n0.06\n")
+
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("refs_as_denom")
+        assert (
+            recipe.to_sql()
+            == """SELECT CASE
+           WHEN (count(*) = 0) THEN NULL
+           ELSE 12 / CAST(count(*) AS FLOAT)
+       END AS refs_as_denom
+FROM scores_with_nulls"""
+        )
+        self.assert_recipe_csv(recipe, "refs_as_denom\n2.0\n")
+
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("math")
+        assert (
+            recipe.to_sql()
+            == """SELECT CASE
+           WHEN (count(*) = 0) THEN NULL
+           ELSE CAST(count(*) AS FLOAT) / CAST(count(*) AS FLOAT)
+       END + 2.5 AS math
+FROM scores_with_nulls"""
+        )
+        self.assert_recipe_csv(recipe, "math\n3.5\n")
+
+        recipe = Recipe(shelf=shelf, session=self.session).metrics("parentheses")
+        assert (
+            recipe.to_sql()
+            == """SELECT CASE
+           WHEN (count(*) + 6.0 = 0) THEN NULL
+           ELSE CAST(count(*) AS FLOAT) / CAST(count(*) + 6.0 AS FLOAT)
+       END AS parentheses
+FROM scores_with_nulls"""
+        )
+        self.assert_recipe_csv(recipe, "parentheses\n0.5\n")
+
+    def test_selectables(self):
+        """Test parsed field definitions built on top of other selectables"""
+        shelf = self.create_shelf(
+            """
+_version: 2
+username:
+    kind: Dimension
+    field: username
+count_star:
+    kind: Metric
+    field: "count(*)"
+""",
+            ScoresWithNulls,
+        )
+        recipe = (
+            Recipe(shelf=shelf, session=self.session)
+            .dimensions("username")
+            .metrics("count_star")
+        )
+        assert (
+            recipe.to_sql()
+            == """SELECT scores_with_nulls.username AS username,
+       count(*) AS count_star
+FROM scores_with_nulls
+GROUP BY username"""
+        )
+        self.assert_recipe_csv(
+            recipe,
+            """username,count_star,username_id
+annika,2,annika
+chip,3,chip
+chris,1,chris
+""",
+        )
+
+        # Build a recipe using the first recipe
+        shelf2 = self.create_shelf(
+            """
+_version: 2
+count_star:
+    kind: Metric
+    field: "count(*)"
+max_username:
+    kind: Metric
+    field: "max(username)"        
+""",
+            recipe,
+        )
+        recipe2 = Recipe(shelf=shelf2, session=self.session).metrics(
+            "count_star", "max_username"
+        )
+        print("two")
+        assert (
+            recipe2.to_sql()
+            == """SELECT count(*) AS count_star,
+       max(anon_1.username) AS max_username
+FROM
+  (SELECT scores_with_nulls.username AS username,
+          count(*) AS count_star
+   FROM scores_with_nulls
+   GROUP BY username) AS anon_1"""
+        )
+        self.assert_recipe_csv(recipe2, "count_star,max_username\n3,chris\n")
+
+
+class TestParsedIntellligentDates(object):
+    def setup(self):
+        self.session = oven.Session()
+
+    def assert_recipe_csv(self, recipe, csv_text):
+        assert recipe.dataset.export("csv", lineterminator=text_type("\n")) == csv_text
+
+    def parsed_shelf(self, config):
+        """Load a file from the sample ingredients.yaml files."""
+        return Shelf.from_validated_yaml(config, DateTester)
+
+    def test_is_current_year(self):
+        """Test current year with a variety of spacing and capitalization"""
+
+        data = [
+            "is current year",
+            "is   current\t  \t year",
+            " IS  CURRENT YEAR",
+            "is Current  Year  ",
+        ]
+
+        for is_current_year in data:
+            shelf = self.parsed_shelf(
+                """
+_version: 2
+test:
+    kind: Metric
+    field: "if(dt {}, count, 0)"
+""".format(
+                    is_current_year
+                )
+            )
+            recipe = Recipe(shelf=shelf, session=self.session).metrics("test")
+            today = date.today()
+            start_dt = date(today.year, 1, 1)
+            end_dt = start_dt + relativedelta(years=1, days=-1)
+            assert (
+                recipe.to_sql()
+                == """SELECT sum(CASE
+               WHEN (datetester.dt BETWEEN '{}' AND '{}') THEN datetester.count
+               ELSE 0
+           END) AS test
+FROM datetester""".format(
+                    start_dt, end_dt
+                )
+            )
+            self.assert_recipe_csv(recipe, "test\n12\n")
+
+    def test_prior_years(self):
+        """Test current year with a variety of spacing and capitalization"""
+
+        data = ["is prior year ", "is  PREVIOUS year  ", "is Last  Year"]
+
+        for is_prior_year in data:
+            shelf = self.parsed_shelf(
+                """
+_version: 2
+test:
+    kind: Metric
+    field: "if(dt {}, count, 0)"
+""".format(
+                    is_prior_year
+                )
+            )
+            recipe = Recipe(shelf=shelf, session=self.session).metrics("test")
+            today = date.today()
+            start_dt = date(today.year - 1, 1, 1)
+            end_dt = start_dt + relativedelta(years=1, days=-1)
+            assert (
+                recipe.to_sql()
+                == """SELECT sum(CASE
+               WHEN (datetester.dt BETWEEN '{}' AND '{}') THEN datetester.count
+               ELSE 0
+           END) AS test
+FROM datetester""".format(
+                    start_dt, end_dt
+                )
+            )
+            self.assert_recipe_csv(recipe, "test\n12\n")
+
+    def test_ytd(self):
+        """Test current year with a variety of spacing and capitalization"""
+
+        data = [
+            "is current ytd ",
+            "is prior ytd  ",
+            "is this  ytd",
+            "is last ytd",
+            "is next ytd",
+        ]
+
+        unique_sql = set()
+        today = date.today()
+
+        for ytd in data:
+            shelf = self.parsed_shelf(
+                """
+_version: 2
+test:
+    kind: Metric
+    field: "if(dt {}, count, 0)"
+""".format(
+                    ytd
+                )
+            )
+            recipe = Recipe(shelf=shelf, session=self.session).metrics("test")
+            self.assert_recipe_csv(recipe, "test\n{}\n".format(today.month))
+            unique_sql.add(recipe.to_sql())
+        assert len(unique_sql) == 3
+
+    def test_is_not(self):
+        """Test current year with a variety of spacing and capitalization"""
+
+        data = [
+            "is current ytd ",
+            "is prior ytd  ",
+            "is this  ytd",
+            "is last ytd",
+            "is next ytd",
+        ]
+
+        unique_sql = set()
+        today = date.today()
+
+        for ytd in data:
+            shelf = self.parsed_shelf(
+                """
+_version: 2
+test:
+    kind: Metric
+    field: "if(not(dt {}), count, 0)"
+""".format(
+                    ytd
+                )
+            )
+            recipe = Recipe(shelf=shelf, session=self.session).metrics("test")
+            self.assert_recipe_csv(recipe, "test\n{}\n".format(100 - today.month))
+            unique_sql.add(recipe.to_sql())
+        assert len(unique_sql) == 3
+
+    def test_qtr(self):
+        """Quarters are always three months"""
+        data = ["is current qtr", "IS PRIOR Qtr", "Is NEXT QTR"]
+
+        unique_sql = set()
+        today = date.today()
+
+        for ytd in data:
+            shelf = self.parsed_shelf(
+                """
+_version: 2
+test:
+    kind: Metric
+    field: "if(dt {}, count, 0)"
+""".format(
+                    ytd
+                )
+            )
+            recipe = Recipe(shelf=shelf, session=self.session).metrics("test")
+            self.assert_recipe_csv(recipe, "test\n3\n")
+            unique_sql.add(recipe.to_sql())
+        assert len(unique_sql) == 3
